/*!
 * \file
 *
 * \brief <Brief Description>
 *
 * \details
 *
 * \copyright AlphaLoewe
 *
 * \author Richard Lowe
 *
 * \date
 *
 * \version <versionNo> - <change_description>
 *
 * Status: <XX% completed.>
 *
 * \note
 * Test configuration:
 *   MCU:             STM32F407VG
 *   Dev.Board:       EasyMxPro STM
 *   Oscillator:      150Mhz
 *   Ext. Modules:    x
 *   SW:              MikroC 3.3.5
 *
 * \par
 *   <all that matters>
 */
#include "websvr.h"
#include <stddef.h>
#include <stdint.h>
#include <stdbool.h>
#include "__Lib_FAT32.h"
#include "__Time.h"
#include "http.h"

#if AVR

#define CLOCK_VECTOR IVT_ADDR_TIMER1_COMPA
// Webserver definitions
sbit Net_Ethernet_28j60_Rst           at PORTA0_bit;
sbit Net_Ethernet_28j60_CS            at PORTC2_bit;
sbit Net_Ethernet_28j60_Rst_Direction at DDA0_bit;
sbit Net_Ethernet_28j60_CS_Direction  at DDC2_bit;
sbit Net_Ethernet_28j60_INT           at PORTD3_bit;
sbit Net_Ethernet_28j60_INT_Direction at DDD3_bit;

sbit Mmc_Chip_Select           at PORTC4_bit;
sbit Mmc_Chip_Select_Direction at DDC4_bit;

#else

#include "bargraph.h"
#define CLOCK_VECTOR IVT_INT_TIM2
sbit Mmc_Chip_Select           at GPIOD_ODR.B3;
sbit bar_graph_cs              at GPIOD_ODR.B13;
sbit bar_graph_rst             at GPIOC_ODR.B2;

#endif

volatile uint8_t counter = 0;
char txtBuffer[64], svrStatus = 0;
char testRequest1[] = "http://opendap.co-ops.nos.noaa.gov/ioos-dif-sos/SOS?service=SOS&request=GetObservation&version=1.0.0&observedProperty=sea_surface_height_amplitude_due_to_equilibrium_ocean_tide&offering=urn:ioos:station:NOAA.NOS.CO-OPS:9432780&responseFormat=text%2Fcsv&eventTime=2013-08-03T00:00:00Z/2013-09-03T23:59:00Z&dataType=HighLowTidePredictions&unit=Meters";
char testRequest2[] = "http://download.linnrecords.com/test/mp3/tone.aspx";
char testRequest3[] = "http://people.oregonstate.edu/~peterseb/misc/test_files/btest001.pdf";

/*******************
 *  Prototypes
 ******************/
void timer_init( void );
void system_init( void );

void process_form_key_values( key_value_t *values );
void process_file_download( char *fileName );
void process_ntp_time( uint32_t time );
void sysinfo_callback( void );

#if AVR
struct webserver_config_t webConfig =
{
    {0x00,0x14,0xA5,0x76,0x19,0x3A},  // Mac address
    {192,168,1,150},                  // Ip address
    {192,168,1,1},                    // Gateway address
    {255,255,255,0},                  // Subnet
    {192,168,1,1},                    // DNS
    80,                               // HTTP Port
    1374388918,                       // Initial unix timestamp
    process_form_key_values,
    process_file_download,
    process_ntp_time,
    FALSE                              // Interrupt mode enable or disable
};

#else
struct webserver_config_t webConfig =
{
    {0x00,0x14,0xA5,0x76,0x19,0x3E},  // Mac address
    {192,168,1,151},                  // Ip address
    {192,168,1,1},                    // Gateway address
    {255,255,255,0},                  // Subnet
    {192,168,1,1},                    // DNS
    80,                               // HTTP Port
    1374388918,                       // Initial unix timestamp
    process_form_key_values,
    process_file_download,
    process_ntp_time,
    FALSE                             // Interrupt mode enable or disable
};
#endif


void system_init()
{
    timer_init();

    #if AVR
    UART1_Init_Advanced( 38400, _UART_NOPARITY, _UART_ONE_STOPBIT );
    #else
    // Terminal output used for debugging
    UART1_Init_Advanced( 115200, _UART_8_BIT_DATA, _UART_NOPARITY, _UART_ONE_STOPBIT, &_GPIO_MODULE_USART1_PA9_10 );
    #endif
    Delay_ms( 100 );  // Allow some time for the UART to stablilize.
    // Initialize SPI in slow speed setting
    #if AVR
    SPI1_Init_Advanced( _SPI_MASTER, _SPI_FCY_DIV128, _SPI_CLK_LO_LEADING );
    #else
    SPI3_Init_Advanced( _SPI_FPCLK_DIV64,
                        _SPI_MASTER | _SPI_8_BIT | _SPI_CLK_IDLE_LOW |
                        _SPI_FIRST_CLK_EDGE_TRANSITION | _SPI_MSB_FIRST |
                        _SPI_SS_DISABLE | _SPI_SSM_ENABLE | _SPI_SSI_1,
                        &_GPIO_MODULE_SPI3_PC10_11_12 );

    GPIO_Digital_Output( &GPIOD_BASE, _GPIO_PINMASK_13 );
    GPIO_Digital_Output( &GPIOD_BASE, _GPIO_PINMASK_2  );

    bar_graph_init( 3 );
    #endif
    Delay_ms( 10 );

    // Initialize FAT32 library for file download capability
    if( ( FAT32_Init() == 0 ) ){

        #if AVR
        SPI1_Init_Advanced( _SPI_MASTER, _SPI_FCY_DIV2, _SPI_CLK_LO_LEADING );
        #else
        // Fast SPI for faster SDCard access
        SPI3_Init_Advanced( _SPI_FPCLK_DIV2,
                            _SPI_MASTER | _SPI_8_BIT | _SPI_CLK_IDLE_LOW |
                            _SPI_FIRST_CLK_EDGE_TRANSITION | _SPI_MSB_FIRST |
                            _SPI_SS_DISABLE | _SPI_SSM_ENABLE | _SPI_SSI_1,
                            &_GPIO_MODULE_SPI3_PC10_11_12 );
        Delay_ms( 10 );
        #endif

        // Initialize the server
        if( webserver_init( &webConfig ) < 0 )
        {
            UART1_Write_Text( "Hardware failed to initialize\r\n" );
            svrStatus = 0;
        }

        else
        {
            svrStatus = 1;
            // Load the catalog
            webserver_add_staticpage( html_code, "index.htm" , sizeof( html_code ) );
            webserver_add_staticpage( p1, "p1.jpg" , sizeof( p1 ) );
            webserver_add_staticpage( p2, "p2.jpg" , sizeof( p2 ) );
            webserver_add_staticpage( p3, "p3.jpg" , sizeof( p3 ) );
            webserver_add_staticpage( p4, "p4.jpg" , sizeof( p4 ) );
            webserver_add_dynamicpage( sysinfo_js,  "sysinfo.js"  , sysinfo_callback  );
        }

    }

    else
    {
        sprinti( txtBuffer, "Filesystem failed with error: %d\r\n", (int)FAT32_GetError() );
        UART1_Write_Text( txtBuffer );
    }

    #if AVR
    SREG_I_bit = 1;
    #else
    EnableInterrupts();
    #endif
}

/*!
 *  \brief Timer initialization
 *
 *  \note
 *   Timer set to 1 second
 */
void timer_init()
{
    #if AVR
    SREG_I_bit = 1;
    TCCR1B = 0x0D;
    OCR1AH = 0x1E;
    OCR1AL = 0x84;
    TIMSK1 = 0x02;
    #else
    RCC_APB1ENR.TIM2EN = 1;
    TIM2_CR1.CEN = 0;
    TIM2_PSC = 2399;
    TIM2_ARR = 62500;
    NVIC_IntEnable(IVT_INT_TIM2);
    TIM2_DIER.UIE = 1;
    TIM2_CR1.CEN = 1;
    #endif
}

void process_form_key_values( key_value_t *values )
{
   int i = 0;

   while( ( values[i]->key[0] != NULL ) && ( i < MAX_FIELDS ) )
   {
       UART1_Write_Text( "Key value : " );
       UART1_Write_Text( values[i]->key );
       UART1_Write_Text( "\r\n" );
       UART1_Write_Text( "Value : " );
       UART1_Write_Text( values[i]->value );
       UART1_Write_Text( "\r\n" );
       i++;
   }

}

void process_file_download( char *fileName )
{
     UART1_Write_Text( "Received a download!\r\nFile Name: " );
     UART1_Write_Text( fileName );
     UART1_Write_Text( "\r\n" );

     FAT32_Dir();
}

void process_ntp_time( uint32_t time )
{
     TimeStruct ts1;                 // Timestruct that will hold gmt time received from NTP server
     __TIME fTime;                   // Filesystem time
     uint32_t gmtTime = time;// GMT time in ms

     gmtTime += ( 60 * 60 * -8 );         // Offset the time difference.  I am in pacific time zone so -8 hours from GMT

     Time_epochToDate( gmtTime, &ts1 ); // Convert adjusted GMT time to local time and hold in ts1

     fTime.Year = ts1.yy;
     fTime.Month = ts1.mo;
     fTime.Day = ts1.md;
     fTime.Hour = ts1.hh;
     fTime.Minute = ts1.mn;
     fTime.Second = ts1.ss;

     FAT32_SetTime( &fTime );       // Set our filesystem time.  Now our webserver is on GMT and filesystem is on local time

     sprinti( txtBuffer, "Time | Date: %d:%d:%d - %d/%d/%d\r\n", (int)ts1.hh, (int)ts1.mn, (int)ts1.ss, (int)ts1.mo, (int)ts1.md, (int)ts1.yy );
     UART1_Write_Text( txtBuffer );
}

void sysinfo_callback( void )
{
     char txt_buff[12];

     memset( sysinfo_js, 0, 512 );

     strcat( sysinfo_js, "var freeMem = '" );
     IntToStr( rand(), txt_buff );
     strcat( sysinfo_js, Ltrim( txt_buff ) );
     
     strcat( sysinfo_js, " kb';\r\nvar freeSoc = '" );
     IntToStr( rand(), txt_buff );
     strcat( sysinfo_js, Ltrim( txt_buff ) );
     
     strcat( sysinfo_js, " of 65535';\r\nvar sysTemp = '" );
     IntToStr( rand(), txt_buff );
     strcat( sysinfo_js, Ltrim( txt_buff ) );
     
     strcat( sysinfo_js, " F';\r\nvar sysTime = '" );
     IntToStr( rand(), txt_buff );
     strcat( sysinfo_js, Ltrim( txt_buff ) );
     
     strcat( sysinfo_js, "';\r\nvar lastPumpTime = '" );
     IntToStr( rand(), txt_buff );
     strcat( sysinfo_js, Ltrim( txt_buff ) );
     
     strcat( sysinfo_js, "';\r\nvar lastPumpVol = '" );
     IntToStr( rand(), txt_buff );
     strcat( sysinfo_js, Ltrim( txt_buff ) );
     strcat( sysinfo_js, "';" );
     
}


void main()
{
    // Wait for clocks to be stable and ready
    #if AVR
    Delay_us( 120 );
    #else
    while( RCC_CR & ( ( 1 << PLLRDY ) | ( 1 << HSERDY ) ) == 0 );
    #endif

    system_init();
    
    UART1_Write_Text( "System Started\r\n" );

    // while server is ready serve pages
    while( svrStatus )
    {
        webserver_read_packet();

        if( UART1_Data_Ready() )
        {
            switch( UART1_Read() )
            {
                case '1':
                    UART1_Write_Text( "Download Test 1 - Chunked\r\n" );
                    webserver_request_file( testRequest1, 80 );
                    break;
                case '2':
                    UART1_Write_Text( "Download Test 2 - Persistant\r\n" );
                    webserver_request_file( testRequest2, 80 );
                    break;
                case '3':
                    UART1_Write_Text( "Download Test 3 - HTTP 1.0\r\n" );
                    webserver_request_file( testRequest3, 80 );
                    break;
                case 's':
                    UART1_Write_Text( "Webserver Status:\r\n" );
                    webserver_print_status();
                    FAT32_Dir();
                    break;
                case 't':
                    UART1_Write_Text( "Requesting Time\r\n" );
                    webserver_request_NTP( "0.pool.ntp.org" );
                    break;
                case 'f':
                    UART1_Write_Text( "Formating SDCard\r\n" );
                    FAT32_Format( "Happy" );
                    FAT32_Dir();
                    break;
                default:
                    UART1_Write_Text( "Not a recognized key\r\n" );
            }
        }

        if( counter >= 10 )
        {
            FAT32_IncTime( 10 );
            webserver_cleanup();
            counter = 0;
        }

    }
}

/*!
 *  \brief ISR - Increments webserver time sensitive variables
 *
 *  \note
 *   <notes>
 */
void Timer2_interrupt() iv CLOCK_VECTOR
{
    #ifndef AVR
    TIM2_SR.UIF = 0;
    #endif
    counter++;
    webserver_time_update();
}