/*!
 * \file webserver.h
 *
 * \brief Webserver
 *
 * \copyright:
 *   (c) Richard Lowe, 12/23/2012
 * Revision History:
 *   .01 - Working
 *   .02 - Added time request function
 *   .03 - Added number of free sockets function
 *   .04 - Changed method of closing sockets, added DHCP configuration as well as renew functions
 *   .05 - Added rotating sockets contained in array
 *   .06 - Added support for client requests
 *   .07 - Defined socket structure
 *   .08 - Added liked list and dynamic memory for better management of aging sockets
 *   .9  - Implimentation of file download
 *   .95 - Added file add functions
 *
 * Status:
 *   95% completed.
 *
 * \note
 * Test configuration:
 *   MCU:             ATMega1284p
 *   Dev.Board:       AVRPLC16
 *   Oscillator:      8Mhz
 *   Ext. Modules:    x
 *   SW:              MikroC 5.8
 *
 */

/*!
* \defgroup server_group Web Server
*    The AlphaLoewe web server is a full-featured HTTP 1.1 compatible TCP server intended for 8-bit microcontrollers.
*    While it is capable of supporting various file types such as: HTML, JPEG, GIF, JavaScript, Java applets, flash, and CGI.
*    It is not recommended for any of these files to be very large in size. This is due to the 8-bit nature and speed of a microcontroller
*    that is capable to run on AA batteries. Therefore, it is recommended that site support be limited to basic functionality.
*
*    Integrated into the Web server is basic authentication, so that before any webpages are provided the user must first authenticate
*    against a saved username and password. This feature, and many more are available to be configured at the user's discretion.
*
*  @{
*/


#ifndef WEBSVR_H
#define WEBSVR_H

#include <stdint.h>
#include <stdbool.h>
#include <stddef.h>

#define SNTP_PORT        123     /*!< Port that is used for SNTP*/
#define REQUESTPORT      10021   /*!< Port that is used for remote server requests*/

#define MAX_FIELDS       20      /*!< Max number of fields in an individual page */
#define MAX_FIELDS_KEY   25      /*!< Max number of chars in key */
#define MAX_FIELDS_VALUE 25      /*!< Max number of chars in value */
#define MAX_PAGES        17       /*!< Max number of pages available in catalog */

#if AVR
#define MY_MSS_SIZE      128
#define MAX_SOCKET        5
#else
#define MY_MSS_SIZE      512
#define MAX_SOCKET        7
#endif

/********************************
     Globals
********************************/
// Key value from from submissions
typedef struct
{
    // Key - form element title
    char key[MAX_FIELDS_KEY];
    // Value - value found in key
    char value[MAX_FIELDS_VALUE];
} key_value_t;

// Configuration of server
struct webserver_config_t
{
    // MAC address
    char myMacAddr[6];
    // IP address
    char myIpAddr [4];
    // gateway (router) IP address
    char gwIpAddr [4];
    // network mask (for example : 255.255.255.0)
    char ipMask   [4];
    // DNS server IP address
    char dnsIpAddr[4];
    // HTTP Port the server will use
    uint16_t port;
    // Initialization timestamp
    uint32_t epochTime;
    // Form received callback
    void ( *form_key_values )( key_value_t *values );
    // File download callback
    void ( *file_download )( char *fileName );
    // NTP timestamp received callback
    void ( *ntp_time )( uint32_t time );
    // IRQ mode enable / disable
    bool irq_enable;
};

/*****************************
     Prototypes
*****************************/
/*!
 * \brief Initializes webserver
 *
 * \param int mode - Interrupts enabled or disabled
 */
int8_t webserver_init( struct webserver_config_t* config );

/*!
 *  \brief Checks the ethernet registry for available packets to process.
 *
 *  \note
 *   This function can either be polled were driven by interrupt based on the initialization of the Web server.
 */
void webserver_read_packet( void );

/*!
 *  \brief Retrieves NTP timestamp from remote server
 *
 *  \param unsigned char *ntpSrv - web address of SNTP server e.g. "www.sntp.gov"
 *
 */
int8_t webserver_request_NTP( char* const ntpSrv );

/*!
 *  \brief Gets time from SNTP server
 *
 *  \param unsigned char *ntpSrv - address of SNTP server
 *
 *  \note
 *   This function does not return the server time. It makes the request,
 *   and waits for a packet to be returned with timestamp information.
 */
int8_t webserver_request_file( char* const request, uint16_t port );

/*!
 *  \brief Searches through working sockets and finds old or unused sockets.
 *
 *  \note
 *   This function should be called periodically to clean up rogue sockets that may
 *   have been disconnected but left file handles and or memory allocation behind.
 */
void webserver_cleanup( void );

/*!
 *  \brief Updates timer for DHCP and DNS functions as well as timestamp for GMT string
 *
 *  \note
 *  This function needs to be called every 1 second
 */
void webserver_time_update( void );

/*!
 *  \brief Addes a static webpage to the server catalog
 *
 *  \param const char* page: pointer to static page
 *
 *  \return int8_t index of page added or failure
 *    \retval 0+ = success index, -1 = unsuccessful
 *
 *  \note
 *   <notes>
 */
int8_t webserver_add_staticpage( const char* page, char* page_name, long page_size );

/*!
 *  \brief Addes a dynamic webpage to the server catalog
 *
 *  \param const char* page: pointer to static page
 *  \param char* page_name: string name of page
 *  \param void( *page_upd_callback )( void ): callback that updates the dynamic data
 *
 *  \return int8_t index of page added or failure
 *    \retval 0+ = success index, -1 = unsuccessful
 *
 *  \note
 *   <notes>
 */
int8_t webserver_add_dynamicpage( char* const page, char* page_name, void ( *page_upd_callback )( void ) );

#ifdef DEBUG
void webserver_print_status( void );
#endif

#endif